Components.utils.import("resource://gre/modules/osfile.jsm");

// Do the download work
var openDownloadMain = {
    _bundle: Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService).createBundle("chrome://opendownload/locale/opendownload.properties"),

    tempfile: "",

    openDownload: function() {
        var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
        var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
        var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
        var prefServiceBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

        if (document.getElementById('open-download').selected){
            try {
                // Complete Path
                var OpenDownload_File = openDownloadMain.getTmpFilePath(dialog.mLauncher.suggestedFileName);

                if ((versionChecker.compare(appInfo.platformVersion, "26.0") < 0 && appInfo.name != "Pale Moon") || appInfo.name == "SeaMonkey") {
                    // Pale Moon 27 started to revert its platformVersion to 3.0.something. Bleh.

                    // Send Notification
                    var notify = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
                    notify.notifyObservers(null, "opendownload-adddownload", OpenDownload_File.path + "??" + (new Date()).getTime());

                    // Save (only Gecko < 26, Gecko 26+ uses Downloads.jsm, see below)
                    dialog.mLauncher.saveToDisk(OpenDownload_File, false);
                }

                // Open
                var oFile   = null;
                var srcFile = dialog.mLauncher.source;
                var delFile = true;

                try {
                    oFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsIFile);

                    if ((versionChecker.compare(appInfo.platformVersion, "26.0") >= 0 || appInfo.name == "Pale Moon") && appInfo.name != "SeaMonkey") {
                        Components.utils.import("resource://gre/modules/Downloads.jsm");
                        Components.utils.import("resource://gre/modules/Services.jsm");
                        Components.utils.import("resource://gre/modules/Task.jsm");

                        Task.spawn(function() {
                            let list = yield Downloads.getList(Downloads.ALL);

                            try {
                                let options = {
                                    source: srcFile,
                                    target: openDownloadMain.tempfile,
                                };

                                let d = yield Downloads.createDownload(options);
                                let res = d.whenSucceeded().then(function() {
                                    d.launch(); // run it!
                                });

                                list.add(d);

                                try {
                                    yield d.start();
                                    yield res;
                                } finally {
                                    if (prefServiceBranch.getPrefType('extensions.openDownload2.deleteFinished')) {
                                        // key exists! try to figure if we want to delete the file...
                                        var prefs = prefService.getBranch("extensions.openDownload2.");
                                        delFile = prefs.getBoolPref("deleteFinished");
                                    }

                                    if (delFile) {
                                        // don't let it pollute the list
                                        yield list.remove(d);
                                    }

                                    yield d.finalize(true); // "remove" it
                                }
                            } finally {
                                // do nothing
                            }
                        }).then(null, Components.utils.reportError);
                    }
                } catch(err) {
                    Components.utils.reportError("OpenDownload (108)\n" + openDownloadMain._bundle.GetStringFromName("error.unknown") + "\n" + err);
                }
            } catch(err) {
                Components.utils.reportError("OpenDownload (111)\n" + openDownloadMain._bundle.GetStringFromName("error.unknown") + "\n"+ err);
                return false;
            }

            dialog.mDialog.dialog = null;
            return true;
        }

    },

    // Create Temp file that does not currently exist
    getTmpFilePath: function(sFilename) {
        try {
            // Split filename and extension
            var sExt = "";
            var iDot = "";
            if ((iDot = sFilename.lastIndexOf(".")) > -1) {
                sExt            = sFilename.substring(iDot);
                sFilename       = sFilename.substring(0, iDot);
            }

            // Directory
            var sTmp = "";
            var sPath = "";

            var oFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsIFile);
            var oDir  = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsIFile);

            // read the extensions.openDownload2.downloadPath.
            // if we have it, use it. if not, don't.
            var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
            var prefServiceBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");

            var i = 0;

            if (prefServiceBranch.getPrefType('extensions.openDownload2.downloadPath')) {
                // key exists! use the user-defined dir.
                var prefs = prefService.getBranch("extensions.openDownload2.");
                sTmp = prefs.getCharPref("downloadPath"); // get the pref

                // does the directory exist? if not: change that!
                oDir.initWithPath(sTmp);
                if (!oDir.exists())           oDir.create(oDir.DIRECTORY_TYPE,0755);
                else if (!oDir.isDirectory()) oDir.create(oDir.DIRECTORY_TYPE,0755);

                // make and verify full path
                sPath = OS.Path.join(sTmp, sFilename + sExt);
                // Note: this will throw NS_ERROR_FILE_UNRECOGNIZED_PATH if the path is invalid (which
                //       shouldn't happen anymore, but who knows...). Should be caught by the error
                //       handling below all this here,
                oFile.initWithPath(sPath);

                // enumeration: we can store multiple files with the same name here :-)
                while (oFile.exists()) {
                    i++;
                    sPath = sTmp + sFilename +"["+ i +"]"+ sExt;
                    oFile.initWithPath(sPath);
                }
            }
            else {
                // nah, let's use the default dir.
                // does the directory exist? if not: change that!
                oDir.initWithPath(OS.Constants.Path.tmpDir);
                oDir.append("mozOpenDownload");

                if (!oDir.exists())           oDir.create(oDir.DIRECTORY_TYPE,0755);
                else if (!oDir.isDirectory()) oDir.create(oDir.DIRECTORY_TYPE,0755);

                sPath = OS.Path.join(OS.Constants.Path.tmpDir,"mozOpenDownload", sFilename + sExt);
                oFile.initWithPath(sPath);

                // enumeration: we can store multiple files with the same name here :-)
                while (oFile.exists()) {
                    i++;
                    sPath = OS.Path.join(OS.Constants.Path.tmpDir,"mozOpenDownload", sFilename +"["+ i +"]"+ sExt);
                    oFile.initWithPath(sPath);
                }
            }

            // Log if we already have more than 3-or-something copies of the file
            if (i > 3) {
                var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].
                    getService(Components.interfaces.nsIConsoleService);

                aConsoleService.logStringMessage("OpenDownload² found several copies of the newly downloaded file. You should clean up the Downloaded directory!");
            }

            openDownloadMain.tempfile = sPath;

            return oFile;

        } catch(err) { Components.utils.reportError("OpenDownload (900)\n" + err); }
    }
}
